% function [nF,Tests]=BaiNg(x,rmax)       Computes test of number of factors according to Bai-Ng (2002)
% BaiNg.m
% Input:
%   x: TxN matrix with the data
%   rmax: maximum number of factors to test (must be lower than min(T,N))
% Output:
%   nF: number of factors, 3-row vector with Bai&Ng's PC1, Pc2 and PC3 tests
%   Tests (optional): IC tests from which number of factors is estimated
% NB: BEWARE! Data is standardised. (Only getting rid of the mean is needed.)
function [nF,Tests]=BaiNg(x,rmax)
[T,N]=size(x);
if nargin<2
    rmax=min(T,N);
end
x=standard(x);  % true global x is not overwritten
scale=[log(N*T/(N+T))*(N+T)/(N*T); log(min(N,T))*(N+T)/(N*T); log(N*T)*(N+T)/(N*T)];
test=zeros(size(scale,1),rmax);
[factor,lambda]=tsupca6(x,rmax);
for r=1:rmax
    error=x-factor(:,1:r)*lambda(:,1:r)';
    test(:,r)=log(mean(sum(error.*error)./T))+r*scale;
end
[dummy,nF]=min(test');
nF=nF';
if nargout>1
    Tests=test;
end
