function [fcsts, sel]=forecasts(pmax, y, data, mmax)
%% This function compute individual forecasts
% if only pmax is supplied, then function returns the number of forecasting methods (nm)
% otherwise:
%           input   'y'     is the depentent variable (observations on t_statr +h: t_end+h)
%           input   'data'  is the matrix of predictors organized by columns (observations on t_start:t_end)
%           output  'fcst'  is the 1 x nm vector of forecasts
%           output  'sel'   is k x nm matrix of selected variables - see below for the definition of k=115
%                           the first dimension of sel correspond to [y,z,factors], k=1+dim(data)+m_max=115; 1 is for lag y, dim(data)=109 is for number of variables in the dataset, and m_max=5 is the largest number of factors considered in the selection stage

no_p=size(pmax,2);
if no_p>1
    'pmax must be a scalar'
    dbstop
end
nm=7*no_p; % number of forecasting methods

if nargin==1
    fcsts=nm; % return the number of forecasting methods
    sel=0;
else
%% individual forecasting methods
    sel=NaN(max(pmax)+size(data,2)+mmax,nm);

% (a) AR(1) 
method_no=1;
    %for pm=pmax % AR(p) forecasts
    pm=pmax; % only one choice of pmax is allowed
    % method #1:pmax: AR(p)
        [fcsts(1,method_no), selp]=ARp_fcst(y,pm);  %AR_fcst(y);
        sel(1:selp,method_no)=1; % selected lags of the dependent variable
        method_no=method_no+1;
   % end %p
    
% (b) factor-augmented AR(p)    
    % first get factors
    Te=size(y)-pmax; % sample used for estimation and for extraction of factors
    Ta=size(y); % size of y vector (=Te+pmax) due to lag selection
    z=[y(pmax:Ta),data(pmax:Ta,:)];
    nz=size(z,2);
    nd=size(data,2);
    D.z=z;
    D.mmax=mmax;
    [F1,zst]=get_factors(D);
    [Fmax,zstm]=get_factors_all(D);  
    
    nfs1=size(F1,2); % this is mhat
    nfsm=size(Fmax,2); % this is mmax
    
    
    %for pm=pmax % factor-augmented AR(p) forecasts only mhat factors are considered
    pm=pmax;
    % method #pmax+1:2pmax: AR(1) augmented with mhat1 factors
        %method_no=pmax+p;
     
        [fcsts(1,method_no), selp]=ARpf_fcst(y,F1,pm);  %ARf_fcst(y,F1);
        mf=size(F1,2);
        sel(1:selp,method_no)=1; % selected lags of y
        if mf>0
            sel(max(pmax)+nd+1:max(pmax)+nd+mf,method_no)=1; % factors
        end
        method_no=method_no+1;
   % end %p

% (c-d) method #3&4: Lasso and adaptive-Lasso applied on predictors augmented with mhat factors
    nd=size(data,2);
    Tpmax=size(y,1); Te=Tpmax-pmax;
    %for pm=pmax % Lasso forecasts
        pm=pmax;
        method_noa=method_no+no_p;
        yl=zeros(Te+1,pm);
        %datal=zeros(Te,nd*pm);
        for j=1:pm
            yl(:,j)=y(pmax-j+1:Tpmax-j+1,1); %Te+1 
            %datal(:,(j-1)*nd+1:nd*j)=data(pmax-j+1:Tpmax-j,:);
        end
        datal=data(pmax:Tpmax,:); %Te+1
        
        dataf=[yl,datal,F1(1:Te+1,:)];    
       [fcsts(1,method_no), sellasso, fcsts(1,method_noa), sellassoa]=Lassof_fcst(y(pmax:Tpmax,1),dataf); 
        %assign selected variables
        sel(1:pm,method_no)=sellasso(1:pm);
        sel(1:pm,method_noa)=sellassoa(1:pm);
        
            sel(max(pmax)+1:max(pmax)+nd,method_no)=sellasso(pm+1:pm+nd);
            sel(max(pmax)+1:max(pmax)+nd,method_noa)=sellassoa(pm+1:pm+nd);
        
        sel(max(pmax)+nd+1:max(pmax)+nd+mf,method_no)=sellasso(pm+nd+1:pm+nd+mf);
        sel(max(pmax)+nd+1:max(pmax)+nd+mf,method_noa)=sellassoa(pm+nd+1:pm+nd+mf);
        method_no=method_no+1;
  %  end %p
    
 

 % (e-g) method #5-7: OCMT conditional on mhat factors   
    selF1=false(nfs1,1);
    selF1all=true(nfs1,1);
    apom=[1:nfs1]';
    selF1allind=apom;

    delta1=1;   delta2=2;

    pval=0.01;
    method_no=method_noa+1; 
    %for p=pmax
    p=pmax;
        [fcsts(1,method_no), sel(:,method_no)]=OCMTpf_fcst(y,data(pmax:Tpmax,:),F1(:,selF1all), pval, delta1, delta2,p, pmax, selF1allind);
        method_no=method_no+1; 
    %end %p
    
    pval=0.05;
    %for p=pmax
    p=pmax;
        [fcsts(1,method_no), sel(:,method_no)]=OCMTpf_fcst(y,data(pmax:Tpmax,:),F1(:,selF1all), pval, delta1, delta2,p, pmax, selF1allind);     
        method_no=method_no+1;  
    %end
    pval=0.1;
    %for p=pmax
    p=pmax;
        [fcsts(1,method_no), sel(:,method_no)]=OCMTpf_fcst(y,data(pmax:Tpmax,:),F1(:,selF1all), pval, delta1, delta2,p, pmax, selF1allind); 
        method_no=method_no+1; 
    %end

end

return

%% local functions

function [yf, selp]=ARp_fcst(y,pmax)
    Tpmax=size(y,1); Te=Tpmax-pmax; 
    if pmax==0
        dbstop;
    end
    b=NaN(pmax+1,pmax);
    for p=1:pmax
        yd=y(pmax+1:Tpmax,1);
        yl=zeros(Te,p);
        for j=1:p
            yl(:,j)=y(pmax-j+1:Tpmax-j,1);
        end
        c=ones(Te,1);
        X=[yl,c];
        b(1:p+1,p)=X\yd;
        e=yd-X*b(1:p+1,p);
        se=e'*e/Te;
        k=p+1;
        SBC(p)=Te*log(se)+k*log(Te);
    end
    [minsbc,selp]=min(SBC);
        
    % use the selected lag selp    
    yL=zeros(1,selp);
    for j=1:selp
        yL(1,j)=y(Tpmax-j+1,1);
    end
    yf=b(1:selp,selp)'*yL'+b(selp+1,selp);
return

function [yf, selp]=ARpf_fcst(y,F,pmax)

nf=size(F,2);
if nf==0 %AR 1 forecasts
    [yf, selp]=ARp_fcst(y,p);
else
   Tpmax=size(y,1); Te=Tpmax-pmax;
   b=NaN(pmax+nf+1,pmax);
   for p=1:pmax
        yd=y(pmax+1:Tpmax,1);
        yl=zeros(Te,p);
        for j=1:p
            yl(:,j)=y(pmax-j+1:Tpmax-j,1);
        end
        Fl=F(1:Te,:);
        c=ones(Te,1);
        X=[yl,Fl,c];
        b(1:p+nf+1,p)=X\yd;
        e=yd-X*b(1:p+1+nf,p);
        se=e'*e/Te;
        k=p+1+nf;
        SBC(p)=Te*log(se)+k*log(Te);
   end
   [minsbc,selp]=min(SBC);
   
   yL=zeros(1,selp);
   for j=1:selp
       yL(1,j)=y(Tpmax-j+1,1);
   end   
   yf=b(1:selp,selp)'*yL'+b(selp+1:selp+nf,selp)'*(F(Te+1,:)')+b(selp+nf+1, selp); 
end
return

function [yf, sellasso, yfa, sellassoa]=Lassof_fcst(y,X)
    T=size(y,1);
    %de-mean y
    muy=y'*ones(T,1)/T;
    ydm=y-muy;
    %de-mean and standardize x
    mux=ones(1,T)*X/T;
    Xdm=X-ones(T,1)*mux;
    stdX=diag(Xdm'*Xdm/(T)).^0.5;
    Xstd=Xdm./(ones(T,1)*stdX');

    [betap,lambdav] = pathl1(Xstd(1:T-1,:), ydm(2:T,1));
    id = select_lambda_lasso(Xstd(1:T-1,:), ydm(2:T,1), lambdav);
    beta=betap(id,:)';
    yf=beta'*Xstd(T,:)'+muy;
    ind=not(beta==0);
    sellasso=NaN(size(beta,1),1);
    sellasso(ind)=1;
    
    % next adaptive Lasso
    Xa=Xstd(:,ind)*diag(abs(beta(ind)));
    if sum(ind)>0
        [beta_a,lambdav] = pathl1_ns(Xa(1:T-1,:), ydm(2:T,1));
        % prediction-based selection
        id = select_lambda_lasso(Xa(1:T-1,:), ydm(2:T,1), lambdav);
        betaa=beta_a(id,:)';
        
        yfa=betaa'*Xa(T,:)'+muy;

        inda=abs(betaa)>0;
        n=size(Xstd,2);    
        od=[1:n]';    
        ods1=od(ind);
        ods2=ods1(inda);
    
        indaf=NaN(n,1);
        indaf(ods2)=1;
        sellassoa=indaf;
 % report results
    else
        yfa=yf;
        sellassoa=sellasso;
    end
    

return


function [yf, selOCMT]=OCMTpf_fcst(y,X,F, pval, delta1, delta2, p, pmax, selFind) 
   
    Tpmax=size(y,1); Te=Tpmax-pmax;
    nx=size(X,2);
    % first OCMT conditional on F 
    yl=zeros(Te,p);
    xl=zeros(Te,nx);
    for j=1:p
        yl(:,j)=y(pmax-j+1:Tpmax-j,1);
        %xl(:,(j-1)*nx+1:j*nx)=X(p-j+1:T-j,:);
    end
    xl=X(1:Te,:);
    eX=[yl,xl];
    ind1=OCMT_sel(y(pmax+1:Tpmax,1),eX,F(1:Te,:), pval, delta1, delta2);

    % now forecast
    Xsel=[F(1:Te,:),eX(:,ind1),ones(Te,1)];
    yd=y(pmax+1:Tpmax,1);
    b=Xsel\yd;
    
    yL=zeros(1,p);
    %xL=zeros(1,p*nx);
    for j=1:p
        yL(:,j)=y(Tpmax-j+1,1);
        %xL(:,(j-1)*nx+1:j*nx)=X(T-j+1,:);
    end
    xL=X(Te+1,:);
    exl=[yL,xL];
    
    yf=b'*[F(Te+1,:), exl(1,ind1),1]';
    %[sum(ind1),ind1(end,1)]
    ns=size(F,2)+size(eX,2);
    nF=size(F,2);
    selOCMT=NaN(pmax+nx+5,1);
    % assign selected regressors
    a=NaN(size(ind1(1:p)));
    a(ind1(1:p))=1;
    selOCMT(1:p,1)=a;
    a=NaN(size(ind1(p+1:p+nx)));
    a(ind1(p+1:p+nx))=1;
    selOCMT(pmax+1:pmax+nx,1)=a;
    selOCMT(pmax+nx+selFind,1)=1;
return



