% tsupca       Peforms PCA
% [factor,lambda,ssr,pctvar]=tsupca(x,r,dim) 
% Input
%       x: Txk matrix with k stationary time series for T periods
%       r: number of factors to extract, optional. Default is k.
%       dim: dimension along which to extract factors, ie T- (dim=1) or k-dimension (dim=2) forced, instead of smaller one.
% Returns
%       ssr: sum of squared residuals of fit between vars and factors
%       factor: Txr time series with the PCA score factors.
%       lambda: rxk matrix with PCA loading factors.
%       pctvar: variance explained by factors (%)

% NB: This code calculates [u,s,v]=svd(z./sqrt(t-1),0) s.t. z=u.s.v.sqrt(t-1)
%     in an economic way, i.e. along the smaller dimension
% RM 5/11/2001
function [factor,lambda,ssr,pctvar]=tsupca(x,r,dim)
if nargin<3
    dim=0;
else
    if dim<1 | dim>2
        dim=0;
    end
end
[t,k]=size(x);
if t>k | dim==1
    xx=x'*x./(t-1);
else
    xx=x*x'./(t-1);
end
[u,s]=svd(xx);
if nargout>3
    pctvar=diag(s)./sum(diag(s)).*100;
end
if nargin>1
    if r~=0 & r<min(t,k)
        u=u(:,1:r);
        s=s(1:r,1:r);
        if nargout>3
            pctvar=pctvar(1:r);
        end
    end
end
if t>k | dim==1
    factor=x*u/sqrt(s);
    if nargout>1
        lambda=u*sqrt(s);
    end
else
    factor=u*sqrt(t-1);
    if nargout>1
        lambda=x'*u./sqrt(t-1);
    end
end
if nargout>2
    ssr=sum(sum(diag(xx-u*s*u')))/k;
end
